/*
 *  Copyright 2013, Enguerrand de Rochefort
 * 
 * This file is part of xdat.
 *
 * xdat is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * xdat is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with xdat.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */


package actionListeners.parallelCoordinatesDisplaySettings;

import gui.frames.ChartFrame;
import gui.panels.AxisDisplaySettingsPanel;
import gui.panels.ParallelCoordinatesChartPanel;
import gui.tables.DataSheetTableColumnModel;

import java.awt.Color;
import java.awt.event.*;

import javax.swing.*;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import xdat.*;

import chart.*;


/**
 * ActionListener that is used for the context menu on the {@link ParallelCoordinatesChartPanel}.
 * 
 */
public class ParallelChartContextMenuActionListener 
implements ActionListener
{
	
	/** Flag to enable debug message printing for this class. */
	static final boolean printLog=true;
	
	/** The main Window. */
	private Main mainWindow;
	
	/** The chart Frame. */
	private ChartFrame chartFrame;
	
	/** The Axis currently being edited. 
 	 */
	private Axis axis;
	

	/**
	 * Instantiates a new context menu for an axis on a parallel coordinates chart.
	 *
	 * @param mainWindow the main window
	 * @param chartFrame the chart frame
	 * @param axis the axis
	 */
	public ParallelChartContextMenuActionListener(Main mainWindow, ChartFrame chartFrame, Axis axis)
	{
		this.mainWindow = mainWindow;
		this.chartFrame = chartFrame;
		this.axis = axis;
		
	}

	
	/* (non-Javadoc)
	 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
	 */
	public void actionPerformed(ActionEvent e)
	{
		String actionCommand = e.getActionCommand();
		if(actionCommand.equals("setCurrentFilterAsNewRange"))
		{
			this.axis.setFilterAsNewRange();
			this.chartFrame.repaint();
		}
		else if(actionCommand.equals("resetFilter"))
		{
			this.axis.resetFilters();
			this.chartFrame.repaint();
		}
		else if(actionCommand.equals("autofit"))
		{
			this.axis.autofit();
			this.chartFrame.repaint();
		}
		else if(actionCommand.equals("moveAxisLeft"))
		{
			DataSheetTableColumnModel cm = (DataSheetTableColumnModel)this.mainWindow.getDataSheetTablePanel().getDataTable().getColumnModel();
			int currentIndex = this.mainWindow.getDataSheet().getParameterIndex(this.axis.getName());
			boolean jumpedAxisWasInactive = true;
			while (currentIndex > 0 && jumpedAxisWasInactive)
			{
				jumpedAxisWasInactive = !((ParallelCoordinatesChart)this.chartFrame.getChart()).getAxis(currentIndex-1).isActive();
//				log("jumpedAxisWasInactive "+jumpedAxisWasInactive+" name: "+((ParallelCoordinatesChart)this.chartFrame.getChart()).getAxis(currentIndex-1).getName());
				cm.moveColumn(currentIndex+1, currentIndex);	// column index starts at one, param index at 0
				currentIndex--;
			}	
		}

		else if(actionCommand.equals("moveAxisRight"))
		{
			DataSheetTableColumnModel cm = (DataSheetTableColumnModel)this.mainWindow.getDataSheetTablePanel().getDataTable().getColumnModel();
			int currentIndex = this.mainWindow.getDataSheet().getParameterIndex(this.axis.getName());
			boolean jumpedAxisWasInactive = true;
			while (currentIndex+2 < cm.getColumnCount() && jumpedAxisWasInactive)
			{
				jumpedAxisWasInactive = !((ParallelCoordinatesChart)this.chartFrame.getChart()).getAxis(currentIndex+1).isActive();
//				log("jumpedAxisWasInactive "+jumpedAxisWasInactive+" name: "+((ParallelCoordinatesChart)this.chartFrame.getChart()).getAxis(currentIndex-1).getName());
				cm.moveColumn(currentIndex+1, currentIndex+2);	// column index starts at one, param index at 0
				currentIndex++;
			}	
		}

		else if(actionCommand.equals("hideAxis"))
		{
			this.axis.setActive(false);
			this.chartFrame.repaint();
		}
		else if(actionCommand.equals("addTic"))
		{
			this.axis.setTicCount(this.axis.getTicCount()+1);
			this.chartFrame.repaint();
		}
		else if(actionCommand.equals("removeTic"))
		{
			this.axis.setTicCount(Math.max(2, this.axis.getTicCount()-1));
			this.chartFrame.repaint();
		}
		else if(actionCommand.equals("reduceDistanceThisAxis"))
		{
			this.axis.setWidth(Math.max(0, axis.getWidth() - 10));
			this.chartFrame.repaint();
		}
		else if(actionCommand.equals("increaseDistanceThisAxis"))
		{
			this.axis.setWidth(Math.max(0, axis.getWidth() + 10));
			this.chartFrame.repaint();
		}
		else if(actionCommand.equals("resetAllFilters"))
		{
			ParallelCoordinatesChart chart = this.axis.getChart();
			for(int i=0; i<chart.getAxisCount(); i++)
			{
				chart.getAxis(i).resetFilters();
			}
			this.chartFrame.repaint();
		}
		else if(actionCommand.equals("reduceDistanceAllAxes"))
		{
			ParallelCoordinatesChart chart = this.axis.getChart();
			for(int i=0; i<chart.getAxisCount(); i++)
			{
				chart.getAxis(i).setWidth(Math.max(0, chart.getAxis(i).getWidth() - 10));
			}
			this.chartFrame.repaint();
		}
		else if(actionCommand.equals("increaseDistanceAllAxes"))
		{
			ParallelCoordinatesChart chart = this.axis.getChart();
			for(int i=0; i<chart.getAxisCount(); i++)
			{
				chart.getAxis(i).setWidth(Math.max(0, chart.getAxis(i).getWidth() + 10));
			}
			this.chartFrame.repaint();
		}
		else
		{
			System.out.println(e.getActionCommand());
		}
	}

	/**
	 * Prints debug information to stdout when printLog is set to true.
	 *
	 * @param message the message
	 */
	private void log(String message)
	{
		if(ParallelChartContextMenuActionListener.printLog && Main.isLoggingEnabled())
		{
			System.out.println(this.getClass().getName()+"."+message);
		}
	}

}
